﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using NAudio.Midi;

namespace Guitar
{
    public partial class MelodyEditMainForm : Form
    {
        public IMelodyEditToolbox EditToolbox;
        public INoteLinesForm NoteLines;

        public MainForm ParentMainForm;

        

        public int CurrentStaff = 1;

        public MelodyEditMainForm(MainForm pMainForm)
        {
            ParentMainForm = pMainForm;

            InitializeComponent();

            int xsize = PreferredSize.Width;
            int ysize = PreferredSize.Height;
            int x = (Screen.PrimaryScreen.Bounds.Width / 2) - (xsize / 2);
            int y = 0; // ysize * 2;
            this.SetDesktopBounds(x, y, xsize, ysize);

        
            EditToolbox = new IMelodyEditToolbox();
            EditToolbox.Show();

            NoteLines = new INoteLinesForm(this);
            NoteLines.Show();

            EditToolbox.BringToFront();
            this.BringToFront();
            
            EditToolbox.TopMost = true;
            this.TopMost = true;

            ParentMainForm.WindowState = FormWindowState.Minimized;
        }

        public void MelodyEditMainForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            EditToolbox.Dispose();
            EditToolbox = null;
            NoteLines.Dispose();
            NoteLines = null;
            ParentMainForm.EditMainForm = null;
            ParentMainForm.WindowState = FormWindowState.Normal;
        }

        private void TransparencyBar_ValueChanged(object sender, EventArgs e)
        {
            NoteLines.Opacity = (float)TransparencyBar.Value / 100f;
        }

        public void btnStaffUp_Click(object sender, EventArgs e)
        {
            CurrentStaff++;
            LabelStaffNum.Text = "Current Staff : " + Convert.ToString(CurrentStaff);
            NoteLines.Refresh();
        }

        public void btnStaffDown_Click(object sender, EventArgs e)
        {
            if (CurrentStaff > 1)
            {
                CurrentStaff--;
                LabelStaffNum.Text = "Current Staff : " + Convert.ToString(CurrentStaff);
                NoteLines.Refresh();
            };

        }

        private void btnMelodyEditSave_Click(object sender, EventArgs e)
        {
            if (ParentMainForm.CurrentFile == null) {
                saveFileDialog1.ShowDialog();
                if (saveFileDialog1.FileName.Length > 0)
                {
                    ParentMainForm.CurrentFile = saveFileDialog1.FileName;
                    this.Text = ParentMainForm.CurrentFile;
                };
            };

            if (ParentMainForm.CurrentFile != null)
            {
                ParentMainForm.CurrentMelody.Save(ParentMainForm.CurrentFile);
            };
        }

        private void toolStripButton1_Click(object sender, EventArgs e)
        {
            ParentMainForm.CurrentMelody.PostProcess((int)MelodySize1.Value, (float)MelodySize2.Value);
        }

        private void btnEndPostprocess_Click(object sender, EventArgs e)
        {
            ParentMainForm.CurrentMelody.FinishPostProcess();
        }

        private void btnEditPlay_Click(object sender, EventArgs e) // play
        {
            
            
            MidiOut midi1 = new MidiOut(0);

            

            midi1.Volume = 65535;
            midi1.Send(MidiMessage.ChangePatch(26, 1).RawData);
            ////////////////////////////////////////////////////////////


            ///////////////////////////////////////////////////////////

            int tactcnt = ParentMainForm.CurrentMelody.GetTactCount();

            for (int tact = 1; tact <= tactcnt; tact++)
            {
                for (float start = 1; start < ParentMainForm.CurrentMelody.Ttactsize1 + 1 ; start = start + 0.25f)
                {
                    for (int i = 0; i < ParentMainForm.CurrentMelody.notes.Count; i++) {
                        if (ParentMainForm.CurrentMelody.notes[i].StartTime == start &&
                            ParentMainForm.CurrentMelody.notes[i].Tact == tact && 
                            ParentMainForm.CurrentMelody.notes[i].MidiNote < 128) {

                            midi1.Send(MidiMessage.StartNote(ParentMainForm.CurrentMelody.notes[i].MidiNote, 127, 1).RawData);
                        };
                    };

                    System.Threading.Thread.Sleep(120);
                }; // end of tact pos
            }; // end if tact

            midi1.Close();
        }

        private void MelodySize1_ValueChanged(object sender, EventArgs e)
        {
            ParentMainForm.CurrentMelody.Ttactsize1 = (float)MelodySize1.Value;
        }

        private void MelodySize2_ValueChanged(object sender, EventArgs e)
        {
            ParentMainForm.CurrentMelody.Ttactsize2 = (float)MelodySize2.Value;
        } // end of play

        
    }
}
